// User Agent Changer upgrade
var useragentchangerUpgrade =
{
	// Gets the version
	getVersion: function()
	{
		return "0.7.3";
	},

	// Installs the user agents
	installUserAgents: function()
	{
		// If the user agent directory does not exist
		if(!useragentchangerImporter.getUserAgentDirectoryLocation().exists())
		{
			useragentchangerImporter.createUserAgentDirectory();
		}

		// If the user agent file does not exist
		if(!useragentchangerImporter.getUserAgentFileLocation().exists())
		{
			useragentchangerImporter.createUserAgentFile();
    		useragentchangerImporter.reset();
		}
	},

	// Migrate to version 0.7
	migrateTo07: function()
	{
		var description    = null;
		var userAgent      = null;
		var userAgentCount = useragentchangerPreferences.getIntegerPreference("useragentchanger.user.agents.count");
		var userAgentFile  = null;
		var xmlDocument    = document.implementation.createDocument("", "", null);
		var rootElement    = xmlDocument.createElement("useragentchanger");
	
		// If the user agent directory does not exist
		if(!useragentchangerImporter.getUserAgentDirectoryLocation().exists())
		{
			useragentchangerImporter.createUserAgentDirectory();
		}
		
		// If the user agent file exists
		if(useragentchangerImporter.getUserAgentFileLocation().exists())
		{
			userAgentFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);

			userAgentFile.initWithPath(useragentchangerImporter.getUserAgentFileLocation().path);
		}
		else
		{
			userAgentFile = useragentchangerImporter.createUserAgentFile();
		}
		
		// Loop through the user agents
		for(var i = 1; i <= userAgentCount; i++)
		{
			description = useragentchangerPreferences.getStringPreference("useragentchanger." + i + ".description");
			
			// If the description is set
    	if(description)
    	{
				userAgent = xmlDocument.createElement("useragent");

				userAgent.setAttribute("description", description);

				// If the user agent is set
				if(useragentchangerPreferences.isPreferenceSet("useragentchanger." + i + ".useragent"))
				{
					userAgent.setAttribute("useragent", useragentchangerPreferences.getStringPreference("useragentchanger." + i + ".useragent"));
				}
				else
				{
					userAgent.setAttribute("useragent", "");
				}
		
				// If the app code name is set
				if(useragentchangerPreferences.isPreferenceSet("useragentchanger." + i + ".appcodename"))
				{
					userAgent.setAttribute("appcodename", useragentchangerPreferences.getStringPreference("useragentchanger." + i + ".appcodename"));
				}
				else
				{
					userAgent.setAttribute("appcodename", "");
				}
		
				// If the app name is set
				if(useragentchangerPreferences.isPreferenceSet("useragentchanger." + i + ".appname"))
				{
					userAgent.setAttribute("appname", useragentchangerPreferences.getStringPreference("useragentchanger." + i + ".appname"));
				}
				else
				{
					userAgent.setAttribute("appname", "");
				}
		
				// If the app version is set
				if(useragentchangerPreferences.isPreferenceSet("useragentchanger." + i + ".appversion"))
				{
					userAgent.setAttribute("appversion", useragentchangerPreferences.getStringPreference("useragentchanger." + i + ".appversion"));
				}
				else
				{
					userAgent.setAttribute("appversion", "");
				}
		
				// If the platform is set
				if(useragentchangerPreferences.isPreferenceSet("useragentchanger." + i + ".platform"))
				{
					userAgent.setAttribute("platform", useragentchangerPreferences.getStringPreference("useragentchanger." + i + ".platform"));
				}
				else
				{
					userAgent.setAttribute("platform", "");
				}
		
				// If the vendor is set
				if(useragentchangerPreferences.isPreferenceSet("useragentchanger." + i + ".vendor"))
				{
					userAgent.setAttribute("vendor", useragentchangerPreferences.getStringPreference("useragentchanger." + i + ".vendor"));
				}
				else
				{
					userAgent.setAttribute("vendor", "");
				}
		
				// If the vendor sub is set
				if(useragentchangerPreferences.isPreferenceSet("useragentchanger." + i + ".vendorsub"))
				{
					userAgent.setAttribute("vendorsub", useragentchangerPreferences.getStringPreference("useragentchanger." + i + ".vendorsub"));
				}
				else
				{
					userAgent.setAttribute("vendorsub", "");
				}
			}		

			rootElement.appendChild(userAgent);
			useragentchangerPreferences.deletePreferenceBranch("useragentchanger." + i + ".");
		}
	
		useragentchangerPreferences.deletePreference("useragentchanger.user.agents.count");

		// If old user agents could be migrated
		if(rootElement.childNodes.length > 0)
		{
			var outputStream  = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
			var xmlSerializer = new XMLSerializer();

			xmlDocument.appendChild(rootElement);
	
			outputStream.init(userAgentFile, 0x04 | 0x08 | 0x20, 00644, null);
			xmlSerializer.serializeToStream(xmlDocument, outputStream, "utf-8");
			outputStream.close();
		} 
		else
		{
			useragentchangerImporter.reset();
		}
	},

	// Opens the upgrade page
	openUpgradePage: function()
	{
		var windowContent = window.getBrowser();
	
		windowContent.removeEventListener("load", useragentchangerUpgrade.openUpgradePage, false);

		window.setTimeout(function() { windowContent.selectedTab = windowContent.addTab("https://addons.mozilla.org/en-US/firefox/user/6228078/" + useragentchangerUpgrade.getVersion().replace(/\./g, "") + "/"); }, 0);	
	},

	// Parses the version number
	parseVersion: function(version)
	{
		// If the version is set
		if(version)
		{
			var splitVersion       = version.split(".");
			var parsedVersion      = splitVersion[0] + ".";
			var splitVersionLength = splitVersion.length;
	
			// Loop through the remaining parts of the version
			for(var i = 1; i < splitVersionLength; i++)
			{
				parsedVersion += splitVersion[i];
			}
	
			return parseFloat(parsedVersion);
		}
		else
		{
			return 0;
		}
	},
	
	// Sets the version
	setVersion: function()
	{
		useragentchangerPreferences.setStringPreference("useragentchanger.version", useragentchangerUpgrade.getVersion());
	},
	
	// Upgrades the extension
	upgrade: function()
	{
		var previousVersion = this.parseVersion(useragentchangerPreferences.getStringPreference("useragentchanger.version", true));
		var version         = this.parseVersion(this.getVersion());

		// If the versions do not match
		if(previousVersion != version)
		{
			// If the previous version is less than 0.7
			if(previousVersion < this.parseVersion("0.7"))
			{
				this.migrateTo07();
			}

			this.installUserAgents();

			window.getBrowser().addEventListener("load", useragentchangerUpgrade.openUpgradePage, false);
	
			useragentchangerPreferences.setStringPreference("useragentchanger.version", version);
		}
	}
};
