// User Agent Changer
var useragentchanger = 
{
	// Displays the about dialog
	about: function()
	{
		window.openDialog("chrome://useragentchanger/content/about/about.xul", "useragentchanger-about-dialog", "centerscreen,chrome,modal");
	},

	// Called when a button has been dropped
	buttonDrop: function(event)
	{
		nsDragAndDrop.drop(event, useragentchanger);
	},
	
	// Changes the options
	changeOptions: function()
	{
		var hideMenuPreference = false;
		var menu               = document.getElementById("useragentchanger-menu");
	
		// If the hide menu preference is set
		if(useragentchangerPreferences.isPreferenceSet("useragentchanger.menu.hide"))
		{
			hideMenuPreference = useragentchangerPreferences.getBooleanPreference("useragentchanger.menu.hide", true);
		}
	
		// If the menu exists
		if(menu)
		{
			menu.setAttribute("hidden", hideMenuPreference);
		}
	},
	
	// Finds the selected user agent and returns it's position and description
	findSelectedUserAgent: function()
	{
		var selectedUserAgent = {};
		var userAgent         = null;
		var userAgents        = document.getElementById("useragentchanger-popup-menu").getElementsByAttribute("type", "radio");
		var userAgentsLength  = userAgents.length;
	
		selectedUserAgent.description = "";
		selectedUserAgent.position    = "";
	
		// Loop through the user agents
		for(var i = 0; i < userAgentsLength; i++)
		{
			userAgent = userAgents.item(i);
			
			// If this is the selected user agent
			if(useragentchanger.isSelectedUserAgent(userAgent))
			{
				selectedUserAgent.description = userAgent.getAttribute("label");
				selectedUserAgent.position    = i;
			
				break;
			}
		}
	
		return selectedUserAgent;
	},
	
	// Returns the selected user agents for a particular menu
	getIndividualSelectedUserAgents: function(menu)
	{
		var userAgents = [];

		// If the menu is set
		if(menu)
		{
			var subMenus       = menu.getElementsByTagName("menu");
			var subMenusLength = subMenus.length;
		
			userAgents = userAgents.concat(useragentchangerArray.convertCollectionToArray(menu.getElementsByAttribute("checked", "true")));

			// Loop through the sub menus
			for(var i = 0; i < subMenusLength; i++)
			{
				userAgents = userAgents.concat(useragentchanger.getIndividualSelectedUserAgents(subMenus[i].firstChild));
			}
		}

		return userAgents;
	},
	
	// Returns the selected user agents
	getSelectedUserAgents: function(windowDocument)
	{
		return useragentchanger.getIndividualSelectedUserAgents(windowDocument.getElementById("useragentchanger-popup-menu")).concat(useragentchanger.getIndividualSelectedUserAgents(windowDocument.getElementById("useragentchanger-popup-toolbar")));
	},
	
	// Return the supported flavours
	getSupportedFlavours: function() 
	{
  	var flavourSet = new FlavourSet();

    flavourSet.appendFlavour("text/toolbarwrapper-id/" + document.documentElement.id);

  	return flavourSet;
 	},
   
	// Opens the help
	help: function()
	{
		window.getBrowser().selectedTab = window.getBrowser().addTab("https://addons.mozilla.org/en-US/firefox/user/6228078/");
	},
	
	// Initializes the extension
	initialize: function(event)
	{
		// Try to get the window content
		try
		{
			var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);

			useragentchangerUpgrade.upgrade();
			useragentchangerImporter.import(useragentchangerImporter.importTypeMenu, useragentchangerImporter.getUserAgentFileLocation(), true);
			useragentchanger.initializeDisplay();
	
			// If the observer service is set
			if(observerService)
			{
				observerService.addObserver(useragentchanger, "quit-application-requested", false);
			}
	
			document.getElementById("navigator-toolbox").addEventListener("dragdrop", useragentchanger.buttonDrop, false);
			window.removeEventListener("load", useragentchanger.initialize, false);
		}
		catch(exception)
		{
			useragentchangerLog.log("useragentchanger.initialize", exception);
		}
	},

	// Initializes the display
	initializeDisplay: function()
	{
		var allWindows           = useragentchangerDOM.getAllWindows();
		var allWindowsLength     = allWindows.length;
		var defaultUserAgent     = null;
		var position             = null;
		var userAgentDescription = null;
		var userAgentSet         = false;

		// If the user agent override preference is set
		if(useragentchangerPreferences.isPreferenceSet("general.useragent.override"))
		{
			var selectedUserAgent = useragentchanger.findSelectedUserAgent();

			position             = selectedUserAgent.position;
			userAgentDescription = selectedUserAgent.description;
			userAgentSet         = true;
		}
		else
		{
			defaultUserAgent = useragentchangerStringBundle.getString("defaultUserAgent");
		}

		// Loop through the open windows
		for(var i = 0; i < allWindowsLength; i++)
		{
			// If a user agent is set
			if(userAgentSet)
			{
				useragentchanger.selectUserAgent(allWindows[i].document, position, userAgentDescription);
			}
			else
			{
				useragentchanger.resetUserAgent(allWindows[i].document, defaultUserAgent);
			}
		}		
	},

	// Returns true if this is the selected user agent
	isSelectedUserAgent: function(userAgent)
	{
		// If all the attributes match
		if((useragentchangerPreferences.getStringPreference("general.useragent.appName", true) == userAgent.getAttribute("useragentchangerappcodename") || (useragentchangerPreferences.getStringPreference("general.useragent.appName", true) == " " && userAgent.getAttribute("useragentchangerappcodename") == "")) &&	
			  useragentchangerPreferences.getStringPreference("general.appname.override", true) == userAgent.getAttribute("useragentchangerappname") &&
			  useragentchangerPreferences.getStringPreference("general.appversion.override", true) == userAgent.getAttribute("useragentchangerappversion") &&
		 	  useragentchangerPreferences.getStringPreference("general.platform.override", true) == userAgent.getAttribute("useragentchangerplatform") &&
			  useragentchangerPreferences.getStringPreference("general.useragent.override", true) == userAgent.getAttribute("useragentchangeruseragent") &&
			  useragentchangerPreferences.getStringPreference("general.useragent.vendor", true) == userAgent.getAttribute("useragentchangervendor") &&
			  useragentchangerPreferences.getStringPreference("general.useragent.vendorSub", true) == userAgent.getAttribute("useragentchangervendorsub"))
		{
			return true;
		}
	
		return false;
	},

	// Observes quits
	observe: function(subject, topic, data)
	{
		// If the reset on close preference is not set or is set to true
		if(!useragentchangerPreferences.isPreferenceSet("useragentchanger.reset.onclose") || useragentchangerPreferences.getBooleanPreference("useragentchanger.reset.onclose", true))
		{
			useragentchanger.reset();
		}

		return false;
	},
	
	// Called when a button has been dropped
	onDrop: function (event, transferData, session) 
	{
		// If the User Agent Changer button was dropped
  	if(transferData.data == "useragentchanger-button")
  	{
			useragentchangerImporter.import(useragentchangerImporter.importTypeMenu, useragentchangerImporter.getUserAgentFileLocation(), true);
			useragentchanger.initializeDisplay();    		
  	}
	},
	
	// Opens a toolbar button automatically if another toolbar button is open on the toolbar
	openToolbarButton: function(currentToolbarButton)
	{
		// If the toolbar button is set and is not open
		if(currentToolbarButton && !currentToolbarButton.open)
		{
			var toolbarButton        = null;
			var toolbarButtons       = currentToolbarButton.parentNode.getElementsByTagName("toolbarbutton");
			var toolbarButtonsLength = toolbarButtons.length;
	
			// Loop through the toolbar buttons
			for(var i = 0; i < toolbarButtonsLength; i++)
			{
				toolbarButton = toolbarButtons.item(i);
	
				// If the toolbar button is set, is not the same toolbar button and is open
				if(toolbarButton && toolbarButton != currentToolbarButton && toolbarButton.open)
				{
					toolbarButton.open        = false;
					currentToolbarButton.open = true;
	
					break;
				}
			}
		}
	},
	
	// Displays the options dialog
	options: function()
	{
		window.openDialog("chrome://useragentchanger/content/options/options.xul", "useragentchanger-options-dialog", "centerscreen,chrome,modal,resizable");
	
		useragentchanger.changeOptions();
	},
	
	// Resets the user agent
	reset: function()
	{
		var allWindows       = useragentchangerDOM.getAllWindows();
		var allWindowsLength = allWindows.length;
		var defaultUserAgent = useragentchangerStringBundle.getString("defaultUserAgent");

		// If an override app code name is set
		if(useragentchangerPreferences.isPreferenceSet("general.useragent.appName"))
		{
			useragentchangerPreferences.deletePreference("general.useragent.appName");
		}
		
		// If an override app name is set
		if(useragentchangerPreferences.isPreferenceSet("general.appname.override"))
		{
			useragentchangerPreferences.deletePreference("general.appname.override");
		}
	
		// If an override app version is set
		if(useragentchangerPreferences.isPreferenceSet("general.appversion.override"))
		{
			useragentchangerPreferences.deletePreference("general.appversion.override");
		}
	
		// If an override platform is set
		if(useragentchangerPreferences.isPreferenceSet("general.platform.override"))
		{
			useragentchangerPreferences.deletePreference("general.platform.override");
		}
	
		// If an override user agent is set
		if(useragentchangerPreferences.isPreferenceSet("general.useragent.override"))
		{
			useragentchangerPreferences.deletePreference("general.useragent.override");
		}
	
		// If an override vendor is set
		if(useragentchangerPreferences.isPreferenceSet("general.useragent.vendor"))
		{
			useragentchangerPreferences.deletePreference("general.useragent.vendor");
		}
	
		// If an override vendor sub is set
		if(useragentchangerPreferences.isPreferenceSet("general.useragent.vendorSub"))
		{
			useragentchangerPreferences.deletePreference("general.useragent.vendorSub");
		}

		// Loop through the open windows
		for(var i = 0; i < allWindowsLength; i++)
		{
			useragentchanger.resetUserAgent(allWindows[i].document, defaultUserAgent);
		}
	},

	// Resets the user agent for a window
	resetUserAgent: function(windowDocument, defaultUserAgent)
	{
		var defaultMenu              = windowDocument.getElementById("useragentchanger-default-menu");
		var defaultToolbar           = windowDocument.getElementById("useragentchanger-default-toolbar");
		var selectedUserAgent        = null;
		var selectedUserAgents       = useragentchanger.getSelectedUserAgents(windowDocument);
		var selectedUserAgentsLength = selectedUserAgents.length;
		var userAgentButton          = windowDocument.getElementById("useragentchanger-button");
		var userAgentMenu            = windowDocument.getElementById("useragentchanger-menu");

		// If the user agent menu exists
		if(userAgentMenu)
		{
			userAgentMenu.setAttribute("image", "chrome://useragentchanger/skin/default.png");
			userAgentMenu.setAttribute("label", defaultUserAgent);
		}

		// If the user agent button is set
		if(userAgentButton)
		{	
			// If the user agent button has a default attribute
			if(userAgentButton.hasAttribute("default"))
			{
				userAgentButton.removeAttribute("default");
			}		

			userAgentButton.setAttribute("label", defaultUserAgent);
			userAgentButton.setAttribute("tooltiptext", defaultUserAgent);
		}
		
		// Loop through the selected user agents
		for(var i = 0; i < selectedUserAgentsLength; i++)
		{
			selectedUserAgent = selectedUserAgents[i];
			
			// If the selected user agent does not have an id or the id is not the default
			if(!selectedUserAgent.hasAttribute("id") || selectedUserAgent.getAttribute("id").indexOf("useragentchanger-default-") != 0)
			{
				selectedUserAgent.removeAttribute("checked");
			}
		}

		// If the default menu exists
		if(defaultMenu)
		{
			defaultMenu.setAttribute("checked", true);
		}

		// If the default toolbar exists
		if(defaultToolbar)
		{
			defaultToolbar.setAttribute("checked", true);
		}
	},

	// Selects the user agent for a window
	selectUserAgent: function(windowDocument, position, selectedUserAgentDescription)
	{
		var positionMenu             = windowDocument.getElementById("useragentchanger-user-agent-" + position + "-menu");
		var positionToolbar          = windowDocument.getElementById("useragentchanger-user-agent-" + position + "-toolbar");
		var selectedUserAgent        = null;
		var selectedUserAgents       = useragentchanger.getSelectedUserAgents(windowDocument);
		var selectedUserAgentsLength = selectedUserAgents.length;
		var userAgentButton          = windowDocument.getElementById("useragentchanger-button");
		var userAgentMenu            = windowDocument.getElementById("useragentchanger-menu");

		// If the user agent menu exists
		if(userAgentMenu)
		{
			userAgentMenu.setAttribute("image", "chrome://useragentchanger/skin/non-default.png");

			// If the selected user agent description is set			
			if(selectedUserAgentDescription)
			{
				userAgentMenu.setAttribute("label", selectedUserAgentDescription);
			}
		}

		// If the user agent button is set
		if(userAgentButton)
		{	
			userAgentButton.setAttribute("default", "false");

			// If the selected user agent description is set			
			if(selectedUserAgentDescription)
			{
				userAgentButton.setAttribute("label", selectedUserAgentDescription);
				userAgentButton.setAttribute("tooltiptext", selectedUserAgentDescription);
			}
		}
		
		// Loop through the selected user agents
		for(var i = 0; i < selectedUserAgentsLength; i++)
		{
			selectedUserAgent = selectedUserAgents[i];
			
			// If the selected user agent does not have an id or the id does not match the position
			if(!selectedUserAgent.hasAttribute("id") || selectedUserAgent.getAttribute("id").indexOf("useragentchanger-user-agent-" + position + "-") != 0)
			{
				selectedUserAgent.removeAttribute("checked");
			}
		}

		// If the position menu exists
		if(positionMenu)
		{
			positionMenu.setAttribute("checked", true);
		}

		// If the position toolbar exists
		if(positionToolbar)
		{
			positionToolbar.setAttribute("checked", true);
		}
	},
	
	// Switches the user agent
	switchUserAgent: function(userAgent)
	{
		var allWindows           = useragentchangerDOM.getAllWindows();
		var allWindowsLength     = allWindows.length;
		var appCodeName          = userAgent.getAttribute("useragentchangerappcodename");
		var position             = userAgent.getAttribute("useragentchangerposition");
		var userAgentDescription = userAgent.getAttribute("label");
		
		// If the app code name is not set add a space since an empty app code name is ignored
		if(!appCodeName)
		{
			appCodeName = " ";
		}
		
		useragentchangerPreferences.setStringPreference("general.useragent.appName", appCodeName);
		useragentchangerPreferences.setStringPreference("general.appname.override", userAgent.getAttribute("useragentchangerappname"));
		useragentchangerPreferences.setStringPreference("general.appversion.override", userAgent.getAttribute("useragentchangerappversion"));
		useragentchangerPreferences.setStringPreference("general.platform.override", userAgent.getAttribute("useragentchangerplatform"));
		useragentchangerPreferences.setStringPreference("general.useragent.override", userAgent.getAttribute("useragentchangeruseragent"));
		useragentchangerPreferences.setStringPreference("general.useragent.vendor", userAgent.getAttribute("useragentchangervendor"));
		useragentchangerPreferences.setStringPreference("general.useragent.vendorSub", userAgent.getAttribute("useragentchangervendorsub"));

		// Loop through the open windows
		for(var i = 0; i < allWindowsLength; i++)
		{
			useragentchanger.selectUserAgent(allWindows[i].document, position, userAgentDescription);
		}
	},
	
	// Opens the test page
	test: function()
	{
		window.getBrowser().selectedTab = window.getBrowser().addTab("https://addons.mozilla.org/en-US/firefox/user/6228078/");
	},
	
	// Uninitializes the extension
	uninitialize: function(event)
	{
		// Try to get the window content
		try
		{
			var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
	
			// If the observer service is set
			if(observerService)
			{
				observerService.removeObserver(useragentchanger, "quit-application-requested", false);
			}
	
			// If the reset on close preference is not set or is set to true
			if(!useragentchangerPreferences.isPreferenceSet("useragentchanger.reset.onclose") || useragentchangerPreferences.getBooleanPreference("useragentchanger.reset.onclose", true))
			{
				var allWindows  = useragentchangerDOM.getAllWindows();
				var windowCount = allWindows.length;
	
				// If this is the last window closing
				if(windowCount == 0)
				{
					useragentchanger.reset();
				}
			}
	
			document.getElementById("navigator-toolbox").removeEventListener("dragdrop", useragentchanger.buttonDrop, false);	
			window.removeEventListener("close", useragentchanger.uninitialize, false);
		}
		catch(exception)
		{
			useragentchangerLog.log("useragentchanger.uninitialize", exception);
		}
	}
};

window.addEventListener("load", useragentchanger.initialize, false);
window.addEventListener("unload", useragentchanger.uninitialize, false);
